@@ -28,14 +28,15 @@ def remove_hidden_methods(ReportClass, outfile=None):
 
     if os.path.exists(outfile) and os.path.samefile(file, outfile):
         raise Exception("Similar file paths identified!")
-    with open(outfile, 'w') as f:
+    with open(os.path.dirname(file) + "/" + outfile, 'w') as f:
         f.write(source)
 
     module_name = ReportClass.__module__
-    module_name.find(".")
-    module_name = module_name[:module_name.rfind(".")] + "." + os.path.basename(outfile)[:-3]
+    # module_name.find(".")
+    i = module_name.rfind(".")
+    module_name = (module_name[:i] if i >=0 else module_name) + "." + os.path.basename(outfile)[:-3]
+
 
     module = importlib.import_module(module_name)
-
     HiddenReportClass = getattr(module, ReportClass.__name__)
     return outfile, HiddenReportClass
