@@ -21,6 +21,7 @@ def download_docker_images(destination=None):
     zf = zipfile.ZipFile(result)
     zf.extractall(path=ex)
     dockers = [f for f in zf.namelist() if f[-1] == "/" and len( [s for s in f if s == "/"] ) == 3]
+    images = {}
     for f in dockers: # zf.namelist():
         tmp_dir = ex + "/" + f
         if os.path.isdir(tmp_dir):
@@ -31,6 +32,9 @@ def download_docker_images(destination=None):
             else:
                 print("> Extracting docker image", os.path.basename(f[:-1]), "to", dest)
                 shutil.copytree(tmp_dir, dest)
+        images[os.path.basename(os.path.normpath(f))] = dest +"/Dockerfile"
+    return images
+
 
 
 def compile_docker_image(Dockerfile, tag=None, no_cache=False):
@@ -195,4 +199,5 @@ def docker_run_token_file(Dockerfile_location, host_tmp_dir, student_token_file,
     if len(tokens) != 1:
         print("Wrong number of tokens produced:", len(tokens))
         print(out)
-    return tokens[0]
+        # assert False
+    return tokens[0], out
