@@ -5,10 +5,9 @@ import inspect
 import time
 import os
 from unitgrade_private import hidden_gather_upload
-# import sys
+from unitgrade_private.deployment import remove_hidden_methods
 import os
 import glob
-# from pupdb.core import PupDB
 
 data = """
 {{head}}
@@ -42,7 +41,19 @@ def lload(flist, excl):
     s = rmimports(s, excl)  # remove import statements from helper class.
     return s
 
-def setup_grade_file_report(ReportClass, execute=False, obfuscate=False, minify=False, bzip=True, nonlatin=False, source_process_fun=None, with_coverage=True, verbose=True):
+def setup_grade_file_report(ReportClass, execute=False, obfuscate=False, minify=False, bzip=True, nonlatin=False, source_process_fun=None, with_coverage=True, verbose=True,
+                            remove_hidden=False, name_without_hidden=None):
+    if remove_hidden:
+        name = os.path.basename(ReportClass.mfile())
+        if name_without_hidden is None:
+            if name.endswith("_complete.py"):
+                name_without_hidden = name[:-12] + ".py"
+            else:
+                raise Exception("Must spacify name to give new report file")
+
+        fout, Report = remove_hidden_methods(ReportClass, outfile=name_without_hidden)  # Create report3.py without @hide-methods
+        return setup_grade_file_report(Report, remove_hidden=False) # Create report3_grade.py for the students
+
     print("Setting up answers...")
     url = ReportClass.url
     ReportClass.url = None
@@ -76,7 +87,8 @@ def setup_grade_file_report(ReportClass, execute=False, obfuscate=False, minify=
     from unitgrade.evaluate import SequentialTestLoader
     loader = SequentialTestLoader()
     for q, points in report.questions:
-        artifacts['questions'][q.__qualname__] = {'title': q.question_title(), 'tests': {} }
+
+        artifacts['questions'][q.__qualname__] = {'title': q.question_title() if hasattr(q, 'question_title') else         q.__qualname__, 'tests': {} }
         suite = loader.loadTestsFromTestCase(q)
         from unitgrade.framework import classmethod_dashboard
 
@@ -92,17 +104,22 @@ def setup_grade_file_report(ReportClass, execute=False, obfuscate=False, minify=
                                                                    'coverage_files': q()._get_coverage_files(),
                                                                    }
         for t in suite._tests:
-            id = t.cache_id()
-            cf = t._get_coverage_files()
-            cf = [] if cf is None else cf
-            artifacts['questions'][q.__qualname__]['tests'][id] = {'title': t.title,
-                                                                   'artifact_file': os.path.relpath(t._artifact_file(), root_dir), # t._artifact_file(),
-                                                                   'hints': t._get_hints(),
-                                                                   'coverage_files': cf
-                                                                   }
+            try:
+                id = t.cache_id()
+                cf = t._get_coverage_files()
+                cf = [] if cf is None else cf
+                artifacts['questions'][q.__qualname__]['tests'][id] = {'title': t.title,
+                                                                       'artifact_file': os.path.relpath(t._artifact_file(), root_dir), # t._artifact_file(),
+                                                                       'hints': t._get_hints(),
+                                                                       'coverage_files': cf
+                                                                       }
+            except Exception as e:
+                pass
             a = 34
     # s, _ = dict2picklestring(artifacts['questions'])
     db['questions'] = artifacts['questions'] # ('questions', s)
+    if not os.path.isdir(os.path.dirname(report._artifact_file())):
+        os.makedirs(os.path.dirname(report._artifact_file()))
     with open(report._artifact_file(), 'wb') as f:
         pickle.dump(db, f)
 
@@ -157,7 +174,16 @@ def setup_grade_file_report(ReportClass, execute=False, obfuscate=False, minify=
                                                        'payload': picklestring.hex(), #repr(picklestring),
                                                        'token_out': repr(fn[:-3] + "_handin"),
                                                        'head': pyhead})
-    output = fn[:-3] + "_grade.py"
+
+    # if fn[:-3].endswith("_test.py"):
+    #     output = fn[:-8] + "_grade.py"
+    # elif fn.endswith("_tests.py"):
+    #     output = fn[:-9] + "_grade.py"
+    # else:
+    #     output = fn[:-3] + "_grade.py"
+
+    output = test2grade(fn)
+
     print("> Writing student script to", output, "(this script may be shared)")
     # Add the relative location string:
 
@@ -201,10 +227,20 @@ def setup_grade_file_report(ReportClass, execute=False, obfuscate=False, minify=
         print("Testing packed files...")
         fn = inspect.getfile(ReportClass)
         print(fn)
-        s = os.path.basename(fn)[:-3] + "_grade"
+        s = os.path.basename(output)[:-3]
+        # s = os.path.basename(fn)[:-3] + "_grade"
         print(s)
         exec("import " + s)
 
     print("====== EXECUTION AND PACKING OF REPORT IS COMPLETE ======")
     ReportClass.url = url
-    return output
\ No newline at end of file
+    return output
+
+def test2grade(file_with_test):
+    if file_with_test.endswith("_test.py"):
+        grade = file_with_test[:-8] + "_grade.py"
+    elif file_with_test.endswith("_tests.py"):
+        grade = file_with_test[:-9] + "_grade.py"
+    else:
+        grade = file_with_test[:-3] + "_grade.py"
+    return grade
\ No newline at end of file
