@@ -24,7 +24,7 @@ def gather_imports(imp):
             print("im is a list")
             print(im)
         # the __path__ attribute *may* be a string in some cases. I had to fix this.
-        print("path.:",  __import__(m.__name__.split('.')[0]).__path__)
+        # print("path.:",  __import__(m.__name__.split('.')[0]).__path__)
         # top_package = __import__(m.__name__.split('.')[0]).__path__._path[0]
         top_package = __import__(m.__name__.split('.')[0]).__path__[0]
         module_import = False
@@ -89,7 +89,7 @@ def gather_report_source_include(report):
             nimp['report_module_specification'] = module_import
             nimp['name'] = m.__name__
             sources[k] = nimp
-            print(f" * {m.__name__}")
+            print(f" * {str(m.__name__)}")
     return sources
 
 def gather_upload_to_campusnet(report, output_dir=None, token_include_plaintext_source=False):
